#ifndef HANDLER_H
#define HANDLER_H
/*

	handler.h

	Uses either Kent Porter's method from Turbo Technix or code that
	is passed to an assembler.  Define the symbol ASSEMBLY in the file
	that includes this file.

	This file is a template; The handler functions may be altered
	but DO NOT alter anything else.

	It is unnecessary to include mouse.h in the module which includes
	this header.
*/

#ifdef ASSEMBLY
#pragma inline
#endif

// Force code for handler module in mouse.h to be generated

#ifndef HANDLER_MODULE
#define HANDLER_MODULE
#endif  HANDLER_MODULE

// if an assembler is not available, uses the method given by Ken Porter in
// Turbo Technix included here

#include "mouse.h"

// use this function in the code which uses the event record to initialize
// it; makes the assembler dependency transparent

inline void event_setup()
{
#ifdef ASSEMBLY
  event_alloc();
#else
  me = MK_FP(_psp,0x00c0);
#endif ASSEMBLY
}

/////////////////////////////////////////////////////////////////////////////

// Here is the customizable part of this file

#ifdef ASSEMBLY
void far interrupt handler()
{
  me->event=_AX;
  me->status=_BX;
  me->x=_CX;
  me->y=_DX;
  asm {
	mov	sp,bp
	pop	bp
	pop	es
	pop	ds
	pop	di
	pop	si
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	retf
  }
}
#else
void far handler()
{
  mouse_event far *save;
  unsigned a=_AX,b=_BX,c=_CX,d=_DX;

  save=MK_FP(_CS-0x10,0x00c0);
  save->event=a;
  save->status=b;
  save->x=c;
  save->y=d;
}
#endif ASSEMBLY

#endif HANDLER_H