#ifndef MOUSE_H
#define MOUSE_H
/*
	mouse.h

	with mouse.cpp, this is a mouse interface for Borland's Turbo C++.
	similar to Ken Porter's C code in Turbo Technix, but better

	Complies with Microsoft/Logitech spec.
	Logitech functions not covered:
		16 conditional hide mouse cursor
		19 set speed threshold

*/
#define call_mdd	int86(0x33,&inreg,&outreg)
#define lower(x, y)	(x<y)?x:y
#define upper(x, y)	(x>y)?x:y
#define HARDWARE	1
#define SOFTWARE	0
#define LEFT	0
#define RITE	1
#define CENTER	2

#include <dos.h>

class mouse
{
  // internal data
	union REGS inreg,outreg;

	int exists,		// TRUE if mouse present
	    buttons;		// # of buttons returned by mouse reset
        int hiding;
	int button_status,      // bits 0-2 on if respective button down
	    op_count;           // # times button clicked
	int column, row;        // position in pixels
	int hcount, vcount;     // net horizontal, vertical movement


  public:

  // these routines deal with setup and status queries

  // reset mouse driver to default status; called by constructors
	void reset();
  // constructor: called when mouse object activated
	mouse(){reset();hiding=1;}
  // destructor: called when mouse deactivated, less programming hassle!
	~mouse(){reset();}

  // these routines are the basic mouse control functions

	void show();		//show cursor; will not bomb if repeat call
	void hide();            //hide cursor; robust like show()
	void getpos();		//get cursor position (pixels), button status
	void setpos(int newcol, int newrow);	//set cursor position
	void pressed(int button);	//records times pressed, etc
	void released(int button);	//records times released, etc
	void xrange(int min, int max);	//set valid ranges, restrict cursor to
	void yrange(int min, int max);  //valid range after called
	void graph_cursor(int h_hot, int v_hot, unsigned mask_seg, 
			  unsigned mask_ofs);
	void text_cursor(int c_type, unsigned arg1, unsigned arg2);
	void motion();		//get horizontal, vertical net motion
	void install_task(unsigned mask, unsigned task_seg, unsigned task_ofs);
	void pen_on();		//toggle light pen operation
	void pen_off();
	void ratio(int horiz, int vert);//set mickey to pixel ratio
					//ratio is R/8, def. 16 vert, 8 horiz
  // internal data access predicates:

  // installation / equipment check
	int installed()		{return exists;}
	int num_buttons()	{return buttons;}
  // status of buttons - not real-time unless read right after call to
  // functions which check position
	int l_down()		{return (button_status&1);}
	int r_down()		{return ((button_status>>1)&1);}
	int c_down()		{return ((button_status>>2)&1);}
  // return number of presses/releases of button
  // use like three prevoius functions
	int count()		{return op_count;}
  // absolute cursor position in pixels
	int x()			{return column;}
	int y()			{return row;}
  // net change in axial movement
	int net_x()		{return hcount;}
	int net_y()		{return vcount;}
};

struct mouse_event{
	unsigned event,		//event that occurred
		 status,	//button status
		 x,y;		//position where event occurred
	void reset()	{event=0;}
};

#ifdef HANDLER_MODULE
mouse_event far *me;

inline int event_exists() {return (me!=(struct mouse_event *)0);}
#ifdef ASSEMBLY
void event_alloc() {if (!event_exists()) me = new mouse_event;}
#endif ASSEMBLY

#else
extern mouse_event far *me;
#endif HANDLER_MODULE

#endif MOUSE_H
